#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/time.h>


enum {
    // _xinfreq = 20_000_000,
    _xtlfreq = 20_000_000,
    _clkfreq = 100_000_000,
    DOWNLOAD_BAUD = 230_400,
    DEBUG_BAUD = DOWNLOAD_BAUD,

};



static uint32_t rdfast_test( uint32_t ticks, uint32_t * addr , uint32_t delay)
{
    __asm volatile { // no optimising and enforces Fcache use - Needed to free up the FIFO
        waitx   ticks
        getct   ticks
        rdfast  nonblk, #64
        waitx   delay
        wrbyte  ticks, addr
        getct   addr
        subr    ticks, addr
        sub     ticks, #6 // remove measurement overheads
        sub     ticks, delay // remove measurement overheads
        waitx   #500    // wait for FIFO to fill before resuming hubexec
        ret

nonblk  long    $8000_0000
    }
    return ticks;
}

static uint32_t rdfast_test_rd( uint32_t ticks, uint32_t * addr , uint32_t delay)
{
    __asm volatile { // no optimising and enforces Fcache use - Needed to free up the FIFO
        waitx   ticks
        getct   ticks
        rdfast  nonblk, #64
        waitx   delay
        rdbyte  addr, addr
        getct   addr
        subr    ticks, addr
        sub     ticks, #6 // remove measurement overheads
        sub     ticks, delay
        waitx   #500    // wait for FIFO to fill before resuming hubexec
        ret

nonblk  long    $8000_0000
    }
    return ticks;
}

static uint32_t test( uint32_t ticks, uint32_t * addr )
{
    __asm volatile { // no optimising and enforces Fcache use - Needed to free up the FIFO
        waitx   ticks
        getct   ticks
        wrbyte  ticks, addr
        getct   addr
        subr    ticks, addr
        sub     ticks, #2
    }
    return ticks;
}

static uint32_t test_rd( uint32_t ticks, uint32_t * addr )
{
    __asm volatile { // no optimising and enforces Fcache use - Needed to free up the FIFO
        waitx   ticks
        getct   ticks
        rdbyte  addr, addr
        getct   addr
        subr    ticks, addr
        sub     ticks, #2
    }
    return ticks;
}


void  main( void )
{
    uint32_t  buff[16], i, j, d;

    _waitms( 200 );
    printf( "\n   clkfreq = %d   clkmode = 0x%x\n", _clockfreq(), _clockmode() );

    printf( "\n[REF]   WRBYTE without RDFAST\n          0   1   2   3   4   5   6   7   8   9\n        ========================================" );
    
    for( j = 0; j < 16; j++ )
    {
        printf( "\n%6x: ", &buff[j] );
        for( i = 0; i < 10; i++ )
            printf( " %2d ", test( i, &buff[j] ) );
    }
    printf( "\n" );

    for (d=0; d<46; d+=2)
    {
        printf( "\n[Separation %d]   WRBYTE after RDFAST\n          0   1   2   3   4   5   6   7   8   9\n        ========================================",d+2 );
        for( j = 0; j < 16; j++ )
        {
            printf( "\n%6x: ", &buff[j] );
            for( i = 0; i < 10; i++ )
                printf( " %2d ", rdfast_test( i, &buff[j], d ));
        }
    }
    printf( "\n----------------------------------------------------------------------------------" );

    printf( "\n[REF]   RDBYTE without RDFAST\n          0   1   2   3   4   5   6   7   8   9\n        ========================================" );
    
    for( j = 0; j < 16; j++ )
    {
        printf( "\n%6x: ", &buff[j] );
        for( i = 0; i < 10; i++ )
            printf( " %2d ", test_rd( i, &buff[j] ) );
    }
    printf( "\n" );

    for (d=0; d<46; d+=2)
    {
        printf( "\n[Separation %d]   RDBYTE after RDFAST\n          0   1   2   3   4   5   6   7   8   9\n        ========================================",d+2 );
        for( j = 0; j < 16; j++ )
        {
            printf( "\n%6x: ", &buff[j] );
            for( i = 0; i < 10; i++ )
                printf( " %2d ", rdfast_test_rd( i, &buff[j], d ));
        }
    }
    printf( "\n" );

}
